from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from selenium.common.exceptions import NoSuchElementException
from urllib.request import urlopen
from bs4 import BeautifulSoup
import pandas as pd
import tkinter as tk
from tkinter import filedialog

def user_input():
    root = tk.Tk()
    root.withdraw()
    input_file_path = filedialog.askopenfilename(filetypes=[('.xlsxfiles', '.xlsx')], title='Select GMDN list excel file')
    i = input_file_path.rfind('/')
    output_folder_path = input_file_path[:i+1]
    # output_file_name = pyautogui.prompt(text='Enter the name of the output file:', title='Output filename',
    # default='')
    # output_file_path = output_folder_path + '/' + output_file_name + '.xlsx'

    #input_path = 'C:\\Users\\spilz\\Documents\\GMDN_codes1.xlsx'
    df = pd.read_excel(input_file_path, usecols=['GMDN-Name/ ΕΙΔΙΚΗ ΟΜΑΔΑ'])
    gmdn_codes_list = df['GMDN-Name/ ΕΙΔΙΚΗ ΟΜΑΔΑ'].str[:5].tolist()
    gmdn_terms_list = df['GMDN-Name/ ΕΙΔΙΚΗ ΟΜΑΔΑ'].str[6:].tolist()
    # gmdn = pyautogui.prompt(text='Enter the GMDN code to be cheked:', title='GMDN Code', default='')
    # gmdn = '11413'
    gmdn_dictionary = dict(zip(gmdn_codes_list, gmdn_terms_list))
    return gmdn_dictionary, output_folder_path


def initialization():
    my_username = 'pappous@htcert.com'
    my_password = '2hiGGtxGKbTHe5'
    gmdn_url = 'https://www.gmdnagency.org/Terms/Search'

    # Dictionary that will be filled with all the data of the searched terms
    D = {'Code': [],
         'INBIT Name': [],
         'GMDN Name': [],
         'GMDN Definition': [],
         'Status': [],
         'Created Date': [],
         'Modified Date': [],
         'Modification': [],
         'Term Exists': [],
         'Identical Names': []}

    errors = 0
    error_list = []

    return my_username, my_password, gmdn_url, D, errors, error_list


def first_login():

    # Driver Settings
    options = webdriver.ChromeOptions()
    options.add_experimental_option("excludeSwitches", ["enable-logging"])
    ser = Service("C:\\Program Files (x86)\\chromedriver.exe")
    driver = webdriver.Chrome(service=ser, options=options)
    driver.get(gmdn_url)

    # Enter login information
    username = driver.find_element(by=By.NAME, value="Email")
    password = driver.find_element(by=By.NAME, value="Password")
    username.send_keys(my_username)
    password.send_keys(my_password)
    driver.find_element(by=By.XPATH, value='/html/body/div[2]/form/div[4]/div/input').click()

    return driver


def gmdn_search(search_term):
    # Individual term search -------------------------------------------------------------------------------------------
    search_bar = driver.find_element(by=By.NAME, value="keyword")
    search_bar.send_keys('\b' * 5)  # Delete previous gmdn code
    search_bar.send_keys(gmdn)  # Enter new gmdn code
    driver.find_element(by=By.ID, value='btnSearch').click()
    # driver.find_element(by=By.XPATH, value='/html/body/div[2]/form/div[4]/table/tbody/tr/th/a').click()

    # Find number of results -------------------------------------------------------------------------------------------
    soup = BeautifulSoup(driver.page_source, "html.parser")
    # s = soup.find('div', {'id': 'list'})
    s = soup.find('h4')
    results = s.text.strip()  # results num
    #print(results)

    # ------------------------------------------------------------------------------------------------------------------
    error = 0
    try:
        if results == '1 term(s)':
            print(f'{search_term} exists')
            #time.sleep(0.1)
            driver.find_element(by=By.XPATH, value='/html/body/div[2]/form/div[4]/table/tbody/tr/th/a').click()

            # Find Useful fields (Name, Definition, Code, Status, Created Date, Modified Date)
            soup = BeautifulSoup(driver.page_source, "html.parser")
            s = soup.find_all('div', {'class': 'form-control-static'})

            Term_Details = {'Code': search_term,
                            'INBIT Name': gmdn_dict[search_term],
                            'GMDN Name': s[0].text,
                            'GMDN Definition': s[1].text,
                            'Status': s[3].text,
                            'Created Date': s[4].text,
                            'Modified Date': '',
                            'Modification': '',
                            'Term Exists': 'Yes',
                            'Identical Names': 'Yes' if gmdn_dict[search_term] == s[0].text else 'No'}

            if len(s) >= 6:
                Term_Details['Modified Date'] = s[5].text
                if len(s) >= 7:
                    Term_Details['Modification'] = s[6].text

            Term_Details = {key.strip(): item.strip() for key, item in Term_Details.items()}

        elif results == 'No terms found.':
            print(f'{search_term} does not exist')
            Term_Details = {'Code': search_term,
                            'INBIT Name': gmdn_dict[search_term],
                            'GMDN Name': '',
                            'GMDN Definition': '',
                            'Status': '',
                            'Created Date': '',
                            'Modified Date': '',
                            'Modification': '',
                            'Term Exists': 'No',
                            'Identical Names': ''}

        else:
            print(f'{search_term} term was neither not found nor returned only one result.')
            Term_Details = {'Code': search_term,
                            'INBIT Name': gmdn_dict[search_term],
                            'GMDN Name': '',
                            'GMDN Definition': '',
                            'Status': '',
                            'Created Date': '',
                            'Modified Date': '',
                            'Modification': '',
                            'Term Exists': 'Other',
                            'Identical Names': ''}

        # Add Specific terms data to Final Dictionary
        for k in D.keys():
            D[k].append(Term_Details[k])

    except:
        error = 1
        print(f'Search term {search_term} yielded an error')
        error_list.append(search_term)
        for k in D.keys():
            D[k].append(Term_Details[k])

    return error


def output(data_dict):
    # Extract results to excel -----------------------------------------------------------------------------------------
    df = pd.DataFrame(data_dict)
    #path = 'C:\\Users\\spilz\\Documents\\gmdn_check_results_test.xlsx'
    path = output_folder_path + '/' + 'gmdn_check_results_test.xlsx'
    df2 = pd.DataFrame({'Codes with Error': error_list})
    #path_e = 'C:\\Users\\spilz\\Documents\\gmdn_check_errors_test.xlsx'
    path_e = output_folder_path + '/' + 'gmdn_check_errors_test.xlsx'

    while True:
        try:

            with pd.ExcelWriter(path) as engine:
                df.to_excel(excel_writer=engine, index=False)
            with pd.ExcelWriter(path_e) as engine:
                df2.to_excel(excel_writer=engine, index=False)
            break
        except IOError:
            window = tk.Tk()
            window.title("Close Excel to continue")

            # create a label for the instructions
            instructions_label = tk.Label(window, height=9, width=50,
                                          text="One or more gmdn check result files are open!\n "
                                               "Please close Excel. Then, press Continue.",
                                          font=('Calibri', 12))
            instructions_label.pack()

            # create a button to confirm closing the excel files
            confirm_button = tk.Button(window, text="Continue", command=window.quit)
            confirm_button.pack()

            # run the Tkinter mainloop
            window.mainloop()
            window.withdraw()
            #input("One or more gmdn check result files are open! Please close Excel.Then, press Enter to retry.")

def main():
    # User Input --------------------------------------------------------------------------------------------------------
    gmdn_dict, output_folder_path = user_input()

    # Initialization -----------------------------------------------------------------------------------------------------
    my_username, my_password, gmdn_url, D, errors, error_list = initialization()

    # First Login, driver creation ---------------------------------------------------------------------------------------
    driver = first_login()

    # GMDN check ----------------------------------------------------------------------------------------------------------
    for gmdn in gmdn_dict.keys():
        errors += gmdn_search(gmdn)
        driver.get(gmdn_url)  # Return to search screen
    driver.quit()
    print("Total errors during the check:", str(errors))

    # Extract results to excel --------------------------------------------------------------------------------------------
    output(D)

    # end ----------------------------------------------------------------------------------------------------------------